/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 38 (Figure 1.13)
**
*/

new;
library rpb,pgraph,optmum;

cls;

rndseed 123456;

_reg_PrintIters = 0;
_reg_cov = 0;
__output = 0;

T = 500;
ns = 1000;

rho = 0.20;

let nVector = 2 3 5 10 20;
nRows = rows(nVector);
ml = zeros(ns,nRows);
var = zeros(nRows,1);
var1 = zeros(ns,nRows);
var2 = zeros(ns,nRows);

iter = 1;
do until iter > nRows;
  n = nVector[iter];
  corr = diagrv(rho .* ones(n,n),1.0);

  x = ones(n,1)/n;
  var[iter] = sqrt(x'*corr*x);

  i = 1;
  do until i > ns;
    data = rndmn(0,corr,T);
    {ml[i,iter],corHat} = regCorr1F(data);

    var1[i,iter] = sqrt(x'*diagrv(ml[i,iter] .* ones(n,n),1.0)*x);
    var2[i,iter] = sqrt(x'*corHat*x);

    i = i + 1;
  endo;

  iter = iter + 1;
endo;

var1 = var1./var';
var2 = var2./var';

x  = seqa(0.10,0.30/250,251);
x1 = seqa(0.80,0.40/250,251);
x2 = x1;

d  = zeros(251,nRows);
d1 = zeros(251,nRows);
d2 = zeros(251,nRows);

iter = 1;
do until iter > nRows;
  {d[.,iter],F} = regKernelDensity(ml[.,iter],x);
  {d1[.,iter],F} = regKernelDensity(var1[.,iter],x1);
  {d2[.,iter],F} = regKernelDensity(var2[.,iter],x2);
  iter = iter + 1;
endo;

x0 = x;
d0 = d;
d0[.,5] = missex(d[.,5], abs(d[.,5]) .<= 1e-2);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3|6|3; _plwidth = 10|10|10|10|10; _pcolor = 1|2|3|4|5; 
  _pstype = 8|8|8|8|9; _plctrl = 20|0|0|0|6;
  xlabel("\216\202r\201 (in %)");
  xtics(10,30,5,2);
  ytics(0,30,5,2);
  _plegstr = "Empirical (n = 2)\000n = 3\000n = 5\000n = 10\000n = 20";
  _plegctl = {2 5 6 4};
  graphprt("-c=1 -cf=chap1-2-1-6.eps");
  xy(100*x0,d0);