/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 63 & 64 (Tables 1.14, 1.15 & 1.16)
**
*/

new;
library rpb,pgraph;

cls;

FileName = "sx5e-june-2012";

factor = 1;

PathName = "data\\";
datFileName = PathName $+ FileName;

loadm num_data = ^datFileName;
index_cov = factor * vread(num_data,"cov");
index_weights = vread(num_data,"weights");

loads num_data = ^datFileName;
index_tickers = num_data[.,1];
index_names = num_data[.,3];

index_weights = index_weights/sumc(index_weights);

rndseed 123;
n = rows(index_Weights);

indx_mu = zeros(n,1);

x1 = zeros(n,n);
te1 = zeros(n,1);
size1 = zeros(n,1);
order1 = zeros(n,1);
active1 = ones(n,1);

x2 = zeros(n,n);
all_te2 = zeros(n,n);
te2 = zeros(n,1);
size2 = zeros(n,1);
order2 = zeros(n,1);
active2 = ones(n,1);

x3 = zeros(n,n);
all_te3 = zeros(n,n);
te3 = zeros(n,1);
size3 = zeros(n,1);
order3 = zeros(n,1);
active3 = zeros(n,1);

n_b = 50;
n_x = 17;
number_loop = zeros(3,1);
n_l = zeros(3,1);
var = sqrt(index_weights'index_cov*index_weights);

i = 1;
do until i > n;

// Traditional

  {x1[.,i],mu_x1,te1[i],gamma_x,retcode} = Compute_Sampling_Portfolio(index_weights,indx_mu,index_cov,0,0,0,0,0,active1);
  y = miss(x1[.,i],0);
  indx = minindc(y);
  size1[i] = sumc(x1[.,i] ./= 0);
  order1[i] = indx;
  active1[indx] = 0;

  number_loop[1] = number_loop[1] + 1;

  if size1[i] == n_x;
    n_l[1] = number_loop[1];
  endif;

// Backward

  new_te = zeros(n,1);
  new_x = zeros(n,n);

  j = 1;
  do until j > n;
    new_active = active2;
    if active2[j] == 0;
      new_te[j] = error(0);
      j = j + 1;
      continue;
    endif;
    new_active[j] = 0;
    {new_x[.,j],mu_x,new_te[j],gamma_x,retcode} = Compute_Sampling_Portfolio(index_weights,indx_mu,index_cov,0,0,0,0,0,new_active);  
    number_loop[2] = number_loop[2] + 1;
    j = j + 1;
  endo;

  all_te2[.,i] = new_te;
  indx = minindc(new_te);
  x2[.,i] = new_x[.,indx];
  te2[i] = new_te[indx];
  size2[i] = sumc(x2[.,i] ./= 0);
  order2[i] = indx;
  active2[indx] = 0;

  if size2[i] == n_x;
    n_l[2] = number_loop[2];
  endif;

// Forward

  new_te = zeros(n,1);
  new_x = zeros(n,n);

  j = 1;
  do until j > n;
    new_active = active3;
    if active3[j] == 1;
      new_te[j] = error(0);
      j = j + 1;
      continue;
    endif;
    new_active[j] = 1;
    {new_x[.,j],mu_x,new_te[j],gamma_x,retcode} = Compute_Sampling_Portfolio(index_weights,indx_mu,index_cov,0,0,0,0,0,new_active);
    number_loop[3] = number_loop[3] + 1;
    j = j + 1;
  endo;

  all_te3[.,i] = new_te;
  indx = minindc(new_te);
  x3[.,i] = new_x[.,indx];
  te3[i] = new_te[indx];
  size3[i] = sumc(x3[.,i] ./= 0);
  order3[i] = indx;
  active3[indx] = 1;

  if size3[i] == n_x;
    n_l[3] = number_loop[3];
  endif;

  i = i + 1;
endo;

order0 = sortind(index_weights);

FileName = "chap1-2-4-6.tex";
call DeleteFile(FileName);

Results = ftosa(seqa(n,-1,n),"%lf",3,0)$~index_tickers[order0]$~index_names[order0]$~ftosa(100*index_weights[order0],"%lf",3,2);
print Results;
call saTabular(Results,FileName);
print "------------------------------------------------------------------------------------------------";

te1 = trimr(te1,1,0)|var;
Results = ftosa(seqa(1,1,n),"%lf",3,0)$~index_tickers[order1]$~strtrunc(index_names[order1],30)$~ftosa(100*index_weights[order1],"%lf",3,2)$~ftosa(100*te1,"%lf",3,2);
print Results;
call saTabular(Results,FileName);
print "------------------------------------------------------------------------------------------------";

te2 = trimr(te2,0,1)|var;
Results = ftosa(seqa(1,1,n),"%lf",3,0)$~index_tickers[order2]$~strtrunc(index_names[order2],30)$~ftosa(100*index_weights[order2],"%lf",3,2)$~ftosa(100*te2,"%lf",3,2);
print Results;
call saTabular(Results,FileName);
print "------------------------------------------------------------------------------------------------";

Results = ftosa(seqa(1,1,n),"%lf",3,0)$~index_tickers[order3]$~strtrunc(index_names[order3],30)$~ftosa(100*index_weights[order3],"%lf",3,2)$~ftosa(100*te3,"%lf",3,2);
print Results;
call saTabular(Results,FileName);
print "------------------------------------------------------------------------------------------------";

print ftosa(n_l',"%lf",3,0);

n1 = (n_b-n_x);
n2 = 0.5 * (n_b - n_x) * (n_b + n_x + 1);
n3 = 0.5 * n_x * (2*n_b - n_x + 1);

print ftosa(n1~n2~n3,"%lf",3,0);
print "------------------------------------------------------------------------------------------------";

te1 = 0|trimr(te1,0,1);
size2 = n|trimr(size2,0,1);
te2 = 0|trimr(te2,0,1);
size3 = rev(size3);
te3 = rev(te3);

FileName = "results\\sampling-sx5e-june-2012";
data = size1~te1~size2~te2~size3~te3;

save ^FileName = data;