/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 76 (Table 2.1)
**
*/

new;
library rpb,pgraph;

cls;

let mu = 50 30 20;
mu = mu/1e4;
let sigma = 0.02 0.03 0.01;
let P = 244 135 315;
n = rows(sigma);
let rho =   1.00 
            0.50   1.00
            0.25   0.60   1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';

print "----------------------------------------";

let x = 2 1 1;
n = x .* P;

sigma_n = sqrt(n'cov*n);
mu_n = n'mu;
print ftos(100*mu_n,"mu(n) = %lf",3,3);
print ftos(100*sigma_n,"sigma(n) = %lf",3,3);

alpha = 0.99;
c_VaR = cdfni(alpha);
c_ES = pdfn(cdfni(alpha))./(1-alpha);
VaR = -mu_n + c_VaR .* sigma_n;
ES = -mu_n + c_ES .* sigma_n;

print ftosa(VaR,"VaR = %lf",3,2);
print ftosa(ES,"ES =  %lf",3,2);

print "----------------------------------------";

let alpha[1,4] = 0.90 0.95 0.99 0.995;
c_VaR = cdfni(alpha);
c_ES = pdfn(cdfni(alpha))./(1-alpha);

print ftosa((c_VaR'~c_ES'),"%lf",3,3);

print "----------------------------------------";

let x = 2 1 1;
P0 = sumc(x .* P);
x = (x .* P)/P0;

print "x = ";
print ftosa(100*x,"%lf",3,2); print;

sigma_x = sqrt(x'cov*x);
mu_x = x'mu;
print ftos(100*mu_x,"mu(x) = %lf",3,3);
print ftos(100*sigma_x,"sigma(x) = %lf",3,3);
print ftos(P0,"P (in $) = %lf",3,2); print;

VaR = -mu_x + c_VaR .* sigma_x;
ES = -mu_x + c_ES .* sigma_x;

print ftosa(100*VaR,"%lf",3,2);
print ftosa(P0*VaR,"%lf",3,2);
print ftosa(100*ES,"%lf",3,2);
print ftosa(P0*ES,"%lf",3,2);

print "----------------------------------------";
let x = 2 -4 1;
P0 = sumc(x .* P);
x = (x .* P)/P0;

print "x = ";
print ftosa(100*x,"%lf",3,2); print;

sigma_x = sqrt(x'cov*x);
mu_x = x'mu;
print ftos(100*mu_x,"mu(x) = %lf",3,2);
print ftos(100*sigma_x,"sigma(x) = %lf",3,2);
print ftos(P0,"P (in $) = %lf",3,2); print;

VaR = -mu_x + c_VaR .* sigma_x;
ES = -mu_x + c_ES .* sigma_x;

print ftosa(100*VaR,"%lf",3,2);
print ftosa(P0*VaR,"%lf",3,2);
print ftosa(100*ES,"%lf",3,2);
print ftosa(P0*ES,"%lf",3,2);