/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 96-98 (Table 2.8)
**
*/

new;
library rpb,pgraph;

cls;

PathName = "data\\";
datFileName = PathName $+ "msci-wgbi-djubs-fed"; 
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

data[.,2:4] = Fill_Miss(data[.,2:4],4);
Dates = data[.,1];

Begin_Date = 20090630;
End_Date = 20120630;
data = selif(data[.,2:3],(Dates .>= Begin_Date) .and (Dates .<= End_Date));

dt = 5;
R = data./lagn(data,dt) - 1.0;
R = trimr(R,dt,0);
nT = rows(R);

let weights[2,7] = 0.00 0.10 0.25 0.50 0.75 0.90 1.00
                   1.00 0.90 0.75 0.50 0.25 0.10 0.00;
nIters = cols(weights);

alpha = 0.99;

Loss_Moments = zeros(4,nIters);
z = zeros(nIters,3);
VaR = zeros(nIters,3);
RC = zeros(2,nIters);

stats = zeros(13,nIters);

i = 1;
do until i > nIters;
  x = weights[.,i];
  PnL = R*x; 
  L = -PnL;
  Loss_moments[1,i] = meanc(L);
  Loss_moments[2,i] = (nT-1)/nT*stdc(L)^2;
  Loss_moments[3,i] = Estimate_Skewness(L);
  Loss_moments[4,i] = Estimate_Kurtosis(L) - 3;
  {z[i,1],VaR[i,1]} = Compute_Cornish_Fisher_z(Loss_moments[1,i],sqrt(Loss_moments[2,i]),0,0,alpha);
  {z[i,2],VaR[i,2]} = Compute_Cornish_Fisher_z(Loss_moments[1,i],sqrt(Loss_moments[2,i]),Loss_moments[3,i],Loss_moments[4,i],alpha);
  {VaR[i,3],RC[.,i],databuf} = Compute_Cornish_Fisher_VaR(x,R,alpha);

  M1 = vread(databuf,"M1");
  M2 = vread(databuf,"M2");
  M3 = vread(databuf,"M3");
  M4 = vread(databuf,"M4");

  m_moments = vread(databuf,"moments");
  m_mu = vread(databuf,"mu");
  m_gamma = vread(databuf,"gamma");
  d_gamma = vread(databuf,"d_gamma");

  stats[1:4,i] = m_mu;
  stats[5:6,i] = m_gamma;
  stats[7:10,i] = vec(d_gamma);
  stats[11,i] = vread(databuf,"z");
  stats[12:13,i] = vread(databuf,"d_z");

  i = i + 1;
endo;

print "M1 = ";
print ftosa(1e4*M1,"%lf",3,4);
print "M2 = ";
print ftosa(1e4*M2,"%lf",3,4);
print "M3 = ";
print ftosa(1e6*M3,"%lf",3,4);
print "M4 = ";
print ftosa(1e6*M4,"%lf",3,3);
print "-------------------------------------------------------------";
print "Moments = ";
print ftosa(100*Loss_Moments,"%lf",3,2);
print "-------------------------------------------------------------";
print "Stats = ";
print ftosa(100*stats,"%lf",3,2);
print "-------------------------------------------------------------";
print "VaR = ";
print ftosa(100*VaR',"%lf",3,2);
print "-------------------------------------------------------------";

stats[1:4,.] = Loss_Moments;
stats = weights | stats[1:4,.] | stats[7:13,.] | VaR[.,3]' | RC | VaR[.,1]';
print "Results = ";
print ftosa(100*stats,"%lf",3,2);
