/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 158 (Table 3.1 & Figure 3.1)
**
*/

new;
library rpb,pgraph;

cls;

Load_Data = 0;

if Load_Data;

  xlsFileName = "data\\Equity-Indexes-June-2012.xlsm";
  nLags = 260;

  SelectDate = SpreadSheetReadM(xlsFileName,"A2:A2",1)/1e6;

  nSheet = xlsGetSheetCount(xlsFileName);

  all_Weights = miss(zeros(3000,nSheet-2),0);
  all_RC  = miss(zeros(3000,nSheet-2),0);
  all_names = "" $+ miss(zeros(nSheet-2,1),0);

  i = 3;
  do until i > nSheet;
    str_data = SpreadSheetReadSA(xlsFileName,"A1",i);
    name = strsplitPad(str_data[1,1],1);
    all_names[i-2] = name[1];
    str_data = trimr(str_data,2,0);

    num_data = SpreadSheetReadM(xlsFileName,"B3",i);
    indx = selif(seqa(1,1,rows(num_data)),num_data[.,1] ./= error(0) );
    tickers = str_data[indx,1];
    weights = num_data[indx,1]/100;
  
    indx = selif(seqa(1,1,rows(num_data)),num_data[.,3] ./= error(0) );
    dates   = num_data[indx,3]/1e6;
    prices  = num_data[indx,4:cols(num_data)];

    Prices  = Fill_Miss(Prices,4);
    Rt = Prices ./ lag1(Prices) - 1.0;
    nAssets = cols(Rt);

    indx1 = closest_index(SelectDate,dates);
    indx = seqa(indx1 - nLags + 1,1,nLags);
    Rt = Rt[indx,.];
    cov = 260*vcx(Rt);

    cov = missrv(cov,0);
    if ismiss(cov);
      errorlog "error";
      {cov2,m} = vcx2(Rt,3);
      cov2 = 260 * cov2;
      end;
    endif;

    RC = weights .* (cov*weights);
    RC = RC ./ sumc(RC);
    RC = miss(RC,0);
  
    all_RC[1:rows(RC),i-2] = RC;
    all_Weights[1:rows(Weights),i-2] = Weights; 

    i = i + 1;
  endo;

  databuf = 0;
  databuf = vput(databuf,all_RC,"rc");
  databuf = vput(databuf,all_Weights,"w");
  save results/chap3-1-3-1 = databuf;
  save results/chap3-1-3-1 = all_names;

endif;

loadm databuf = results/chap3-1-3-1;
loads all_names = results/chap3-1-3-1;

all_rc = vread(databuf,"rc");
all_w = vread(databuf,"w");

nIndices = cols(all_w);

Gini = zeros(nIndices,2);

let alpha = 0.10 0.25 0.50 0.75;
nAlpha = rows(alpha);
Results_L_w = zeros(nIndices,nAlpha);
Results_L_rc = zeros(nIndices,nAlpha);

q = seqa(0,1/100,101);
Lorenz_w  = zeros(rows(q),nIndices);
Lorenz_rc = zeros(rows(q),nIndices);

let indx_alpha = 11 26 51 76;

i = 1;
do until i >  nIndices;

  all_data = all_w[.,i]~all_rc[.,i];

  {x,Lorenz_W[.,i],Gini[i,1]} = Lorenz_Curve(all_w[.,i],q);
  Results_L_w[i,.] = Lorenz_W[indx_alpha,i]';
  {x,Lorenz_RC[.,i],Gini[i,2]} = Lorenz_Curve(all_rc[.,i],q);
  Results_L_rc[i,.] = Lorenz_RC[indx_alpha,i]';

  i = i + 1;
endo;

cls;
Results = all_names $~ ftosa(100*Gini[.,1],"%lf",3,1) $~ftosa(100*Results_L_w[.,1:3],"%lf",3,1)
                    $~ ftosa(100*Gini[.,2],"%lf",3,1) $~ftosa(100*Results_L_rc[.,1:3],"%lf",3,1);
indx = sortind(Gini[.,1]);
Results = Results[indx,.];
print Results;

let string selected_indices = "SX5P" "DAX" "IBEX" "INDU" "UKX" "KOSPI";
indx_selected = indsav(selected_indices,all_Names);

graphset;
  begwind;
  window(2,2,0);
 
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|2|4;
  _paxht = 0.25;  _ptitlht = 0.275;  _pnumht = 0.20;

  ytics(0,100,20,2);
  xtics(0,100,20,2);
  _pline = 1~6~0~0~100~100~1~0~5;

  setwind(1);
    title("\216Lorenz curve of weights");
    xy(100*q,100*Lorenz_W[.,indx_selected[1:3]]);
   
  setwind(2);
    xy(100*q,100*Lorenz_W[.,indx_selected[4:6]]);

  setwind(3);
    title("\216Lorenz curve of risk contributions");
    _plegstr = MakeMsgStr(all_Names[indx_selected[1:3]]);
    _plegctl = {2 5 6.5 1.25};
    xy(100*q,100*Lorenz_RC[.,indx_selected[1:3]]);

  setwind(4);
    _plegstr = MakeMsgStr(all_Names[indx_selected[4:6]]);
    xy(100*q,100*Lorenz_RC[.,indx_selected[4:6]]);

   graphprt("-c=1 -cf=chap3-1-3-1.eps");

  endwind;