/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 226 (Figure 4.14)
**
*/

new;
library rpb,pgraph;

cls;

FileName = "Data//sovereign-bonds.fmt";
loadm databuf = ^FileName;
Dates = vread(databuf,"dates");
Spread = vread(databuf,"spread");
Spread_Greece = vread(databuf,"sp-gr");
Duration = vread(databuf,"duration");
Weights_EGBI = vread(databuf,"w(egbi)");
Weights_GDP = vread(databuf,"w(gdp)");
Weights_DEBT = vread(databuf,"w(debt)");
EGBI_index = vread(databuf,"index");
EGBI_Subindex = vread(databuf,"subindex");

FileName = "Results//sovereign-bonds-vol";
loadm vol = ^FileName;

FileName = "Results//sovereign-bonds-rho";
loadm cor = ^FileName;

nCountries = cols(Spread);

Begin_Date = 20071231;
End_Date = 20120630;
nDates = rows(Dates);
indx_Begin_Date = closest_index(Begin_Date,Dates);

rho = zeros(nDates,7);
e = miss(diagrv(ones(11,11),error(0)),0);

t = indx_Begin_Date;
do until t > nDates;
  cor_t = arraytomat(cor[t,.,.]);
  cor_t = cor_t .* e;
  rho_1 = meanc(packr(vec(cor_t)));                           // mean correlation
  rho_2 = meanc(packr(cor_t[.,3]));                           // mean correlation of Finland
  rho_3 = meanc(packr(vec(cor_t[3 5 9,3 5 9])));              // mean correlation between FGN
  rho_4 = meanc(packr(vec(cor_t[6 7 8 10 11,6 7 8 10 11])));  // mean correlation between GIIPS
  rho_5 = minc(packr(vec(cor_t)));                            // min correlation
  rho_6 = maxc(packr(vec(cor_t)));                            // max correlation
  rho_7 = meanc(packr(vec(cor_t[7 8 10 11,7 8 10 11])));      // mean correlation between IIPS

  rho[t,.] = rho_1~rho_2~rho_3~rho_4~rho_5~rho_6~rho_7;

  t = t + 1;
endo;

data = dates~rho;
data = selif(data,data[.,1] .>= Begin_Date .and data[.,1] .<= End_Date);
dates = data[.,1];
rho = data[.,2 4 5 8];
nDates = rows(dates);
rho[1,.] = error(0) * rho[1,.];

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0; _pgrid = {1,1};
  _pltype = 6|3|1|6; _plwidth = 10; _pcolor = 1|2|3|4; _plctrl = 0|0|0|131; _pstype = 8; 
  let xlab = "2008" "" "2009" "" "2010" "" "2011" "" "2012" "";
  asclabel(xlab,0);
  xtics(1,nDates,131,0);
  ytics(20,90,10,2);
  _plegstr = "AC\000FI-DE-NL\000GIIPS\000IIPS";
  _plegctl = {2 5 4.25 1.25};
  graphprt("-c=1 -cf=chap4-3-2-4.eps");
  xy(0,100*rho);


