/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 274 (Figure 6.3)
**
*/

new;
library rpb,pgraph;

cls;

PathName = "data\\";
datFileName = PathName $+ "msci-wgbi-djubs-fed"; 
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

Begin_Date = 19981231;
End_Date = 20111230;

data = selif(data,(data[.,1] .>= Begin_Date) .and (data[.,1] .<= End_Date));
data = Fill_Miss(data,4);

Dates = data[.,1];
nDates = rows(Dates);

Data = Data[.,2 3];
Data = 100 * Data ./ Data[1,.];

{cov,vol,cor} = Rolling_Covariance(Dates,Data,260,1);

let w1 = 0.20 0.80;
let w2 = 0.50 0.50;
let w3 = 0.80 0.20;

rc1 = zeros(nDates,2);
rc2 = zeros(nDates,2);
rc3 = zeros(nDates,2);
vol = zeros(nDates,3);

t = 1;
do until t > nDates;

  Sigma = arraytomat(cov[t,.,.]);

  {vol[t,1],mr,rc,prc} = Compute_Risk_Contribution(w1,Sigma);
  rc1[t,.] = prc';

  {vol[t,2],mr,rc,prc} = Compute_Risk_Contribution(w2,Sigma);
  rc2[t,.] = prc';

  {vol[t,3],mr,rc,prc} = Compute_Risk_Contribution(w3,Sigma);
  rc3[t,.] = prc';

  t = t + 1;
endo;

t = selif(seqa(1,1,nDates),Dates .>= 20000101);
rc1 = rc1[t,.];
rc2 = rc2[t,.];
rc3 = rc3[t,.];
vol = vol[t,.];
nDates = rows(t);
t = seqa(1,1,nDates);

step = 5;
indx_t = seqa(1,step,nDates/step)|nDates;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|2;
  _pcolor = 3|4|5;
  xtics(1,nDates,261*2,2);
  let xlab = "2000" "02" "04" "06" "08" "10" "12";
  asclabel(xlab,0);
  ytics(0,30,5,2);
  _plegstr = "Defensive\000Balanced\000Dynamic";
  _plegctl = {2 5 3.3 4.25};
  graphprt("-c=1 -cf=chap6-1-1-3.eps");
  xy(t[indx_t],100*vol[indx_t,.]);